#region usings

using System;
using System.ComponentModel.Composition;
using System.Collections.Generic;
using IrrKlang;
using IrrVec3D = IrrKlang.Vector3D;
using VVVV.PluginInterfaces.V1;
using VVVV.PluginInterfaces.V2;
using VVVV.Utils.VColor;
using VVVV.Utils.VMath;
using Vector3D = VVVV.Utils.VMath.Vector3D;
using VVVV.Core.Logging;

#endregion usings

namespace VVVV.Nodes
{ 
	#region PluginInfo
	[PluginInfo(Name = "Irrklang", Category = "sound", Version = "1", Help = "Basic template with one value in/out", Tags = "")]
	#endregion PluginInfo
	public class Irrklang : IPluginEvaluate
	{
		#region fields & pins
		[Input("File", DefaultString = "hello c#")]
		ISpread<string> FFile;
        [Input("Pos", DefaultValue = 0)]
		ISpread<Vector3D> FPos;
		[Input("minimal distance", DefaultValue = 1.0)]
		ISpread<float> FmDist;
	    [Input("Playback Speed", DefaultValue = 1.0)]
		ISpread<float> FPlaySpeed;
		[Input("Play Position", DefaultValue = 1.0)]
		ISpread<float> FPlayPos;
		[Input("volume", DefaultValue = 1.0)]
		ISpread<float> FVolume;
		[Input("Play")]
		ISpread<bool> FPlay;
		[Input("Pause")]
		ISpread<bool> FPause;
		[Input("Seek Pos")]
		ISpread<bool> FSeekPos;
        [Input("Loop")]
		ISpread<bool> FLoop;
		[Input("Stop All")]
		ISpread<bool> FStopAll;
		[Input("Update")]
		ISpread<bool> Fupdate;
		
		[Input("Driver", EnumName = "DriverName")]
		IDiffSpread<EnumEntry> FDriver;
        [Output("length")]
		ISpread<double> Flength;
		[Output("finished")]
		ISpread<bool> FFinish;
		[Output("Play Pos")]
		ISpread<double> FCurrentPos;
		ISoundEngine FEngine;	
		Dictionary< int , ISound> FSounds;
        
		//[Import()]
		//ILogger Flogger;
		#endregion fields & pins
	 	
        public Irrklang()
        {
			//Flogger.Log(LogType.Debug, "creating sound engine");
	    	FEngine = new ISoundEngine(); 
	    	FSounds = new Dictionary< int ,ISound >();
	    	//EnumManager.UpdateEnum("MyDynamicEnum", ISoundDeviceList[0], ISoundDeviceList);
			//Flogger.Log(LogType.Debug, "done"); 
					
        }
		    
		//called each frame by vvvv
		public void Evaluate(int SpreadMax)
		{  
		 
		    Flength.SliceCount = SpreadMax;
            FCurrentPos.SliceCount = SpreadMax;
            FFinish.SliceCount = SpreadMax;
			for (int i = 0; i < SpreadMax; i++){
			 
			    if (FPlay[i] & (!FSounds.ContainsKey(i)))
			    {
			 	    FSounds.Add(i,(FEngine.Play3D(FFile[i],(float)FPos[i].x,(float)FPos[i].y,(float)FPos[i].z,FLoop[i])));	
				   
			    }
			    
			    if (FSounds.ContainsKey(i))
		     	{   
				    FSounds[i].Position = new IrrVec3D((float)FPos[i].x,(float)FPos[i].y,(float)FPos[i].z);
				    FSounds[i].MinDistance = FmDist[i];
				    FSounds[i].PlaybackSpeed = FPlaySpeed[i];
				    FSounds[i].Volume = FVolume[i];
				    FSounds[i].Paused = FPause[i];
				    Flength[i] = (FSounds[i].PlayLength)/1000;
				    FCurrentPos[i] = (FSounds[i].PlayPosition)/1000;
				    FFinish[i] = FSounds[i].Finished;
				    if(FSounds[i].Finished)
				    {
				        
				        FSounds.Remove(i);
				        FCurrentPos[i] = 0;
				    }
			    }	
			   
			    if (FSeekPos[i] & FSounds.ContainsKey(i))
			    {
                    FSounds[i].PlayPosition = ((UInt32)FPlayPos[i])*1000;
                }  
           
			    if (FStopAll[i])
				    FEngine.StopAllSounds();
						
		    }
		}
	}
}
